use std::env;
use std::path::PathBuf;

use chrono::{DateTime, Duration, NaiveDateTime, Utc};
use clap::{Parser, Subcommand};
use rusqlite::{params, Connection};

#[derive(Parser)]
#[command(name = "whenst")]
#[command(about = "A simple event countdown CLI")]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// Add a new event
    Add {
        /// Name of the event
        name: String,
        /// The date of the event (YYYY-MM-DD HH:MM)
        date: String,
    },

    /// List all upcoming events
    List {
        /// Show how much time is left until the next event
        #[arg(short, long, default_value_t = false)]
        time: bool,
    },

    /// Show the next upcoming event
    Next {
        /// Show id
        #[arg(short, long, default_value_t = false)]
        id: bool,

        /// Show how much time is left until the next event
        #[arg(short, long, default_value_t = false)]
        time: bool,

        /// Show the event name
        #[arg(short, long, default_value_t = false)]
        name: bool,
    },

    /// Remove an event by ID
    Remove {
        /// The ID of the event to remove
        id: i32,
    },
}

fn init_db() -> rusqlite::Result<rusqlite::Connection> {
    let conn = Connection::open(get_config_path())?;
    conn.execute(
        "CREATE TABLE IF NOT EXISTS events (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT NOT NULL,
            timestamp INTEGER NOT NULL
        )",
        [],
    )?;
    Ok(conn)
}

fn parse_date(date_str: &str) -> Option<i64> {
    NaiveDateTime::parse_from_str(date_str, "%Y-%m-%d %H:%M")
        .ok()
        .map(|dt| dt.and_utc().timestamp())
}

fn main() -> rusqlite::Result<()> {
    let conn = init_db()?;
    let cli = Cli::parse();

    match cli.command {
        Commands::Add { name, date } => {
            let timestamp = match parse_date(&date) {
                Some(timestamp) => timestamp,
                None => {
                    eprintln!("Invalid date format");
                    return Err(rusqlite::Error::InvalidQuery);
                }
            };

            conn.execute(
                "INSERT INTO events (name, timestamp) VALUES (?1, ?2)",
                params![name, timestamp],
            )?;

            println!("Added event: \"{}\" at {}", name, date);
        }
        Commands::List { time: show_time } => {
            let mut stmt =
                conn.prepare("SELECT id, name, timestamp FROM events ORDER BY timestamp ASC")?;
            let events_iter = stmt.query_map([], |row| {
                Ok((
                    row.get::<_, i32>(0)?,
                    row.get::<_, String>(1)?,
                    row.get::<_, i64>(2)?,
                ))
            })?;

            for event in events_iter {
                let (id, name, timestamp) = event?;
                let datetime = DateTime::from_timestamp(timestamp, 0).unwrap();
                let mut total_string =
                    format!("[{}] {} - {}", id, name, datetime.format("%Y-%m-%d %H:%M"));

                if show_time {
                    let remaining = format_time_left(timestamp);
                    total_string.push_str(&format!(" - {}", remaining));
                }

                println!("{}", total_string);
            }
        }
        Commands::Next {
            id: show_id,
            time: show_time,
            name: show_name,
        } => {
            let mut stmt = conn.prepare("SELECT id, name, timestamp FROM events WHERE timestamp > strftime('%s', 'now') ORDER BY timestamp ASC LIMIT 1")?;
            let mut rows = stmt.query([])?;

            if let Some(row) = rows.next()? {
                let id: i32 = row.get(0)?;
                let name: String = row.get(1)?;
                let timestamp: i64 = row.get(2)?;
                let datetime = DateTime::from_timestamp(timestamp, 0).unwrap();

                let mut total_string = String::from("Next event: ");
                if show_id {
                    total_string.push_str(&format!("[{}] ", id));
                };

                if show_name {
                    total_string.push_str(&format!("{} - ", name,));
                }

                if show_time {
                    let remaining = format_time_left(timestamp);
                    total_string.push_str(&remaining.to_string());
                } else {
                    total_string.push_str(&format!("{}", datetime.format("%Y-%m-%d %H:%M")));
                }

                println!("{}", total_string);
            } else {
                println!("No upcoming events.");
            }
        }
        Commands::Remove { id } => {
            let deleted = conn.execute("DELETE FROM events WHERE id = ?1", params![id])?;
            if deleted > 0 {
                println!("Removed event with ID {}", id);
            } else {
                println!("No event found with ID {}", id);
            }
        }
    }

    Ok(())
}

#[inline]
fn get_config_path() -> PathBuf {
    if cfg!(target_os = "windows") {
        let username = env::var("USERNAME").unwrap_or_else(|_| "Default".to_string());
        PathBuf::from(format!(
            "C:\\Users\\{}\\AppData\\Local\\whenst.db",
            username
        ))
    } else {
        let xdg_config_home = env::var("XDG_CONFIG_HOME").ok();
        let home = env::var("HOME").ok();
        let user = env::var("USER").unwrap_or_else(|_| "default".to_string());

        xdg_config_home
            .map(|x| PathBuf::from(format!("{}/whenst.db", x)))
            .or_else(|| home.map(|h| PathBuf::from(format!("{}/.config/whenst.db", h))))
            .unwrap_or_else(|| PathBuf::from(format!("/home/{}/.config/whenst.db", user)))
    }
}

fn format_time_left(event_time: i64) -> String {
    let now = Utc::now().timestamp();
    let duration = event_time - now;

    if duration <= 0 {
        return "Event started!".to_string();
    }

    let duration = Duration::seconds(duration);
    let days = duration.num_days();
    let hours = duration.num_hours() % 24;
    let minutes = duration.num_minutes() % 60;

    if days > 0 {
        format!("{}d {}h left", days, hours)
    } else if hours > 0 {
        format!("{}h {}m left", hours, minutes)
    } else {
        format!("{}m left", minutes)
    }
}
